import { h, Component } from 'preact'
import assign from 'object-assign'

/**
 * Higher order component used to add validation capabilities to a form input.
 * @param {Component} WrappedComponent - preact component to add validation
 * @prop {String|Function} validate - string (not-emtpy, email, checked)
 * or custom function used to validate input. function is called with the value
 * of the field.
 * @prop {Function} onValidityChange - function called with the validity
 * status and message when the valdidity of the input changes.
 * @prop {Boolean} forceValidate - any time this value is set to true, the
 * field will validate, even if its content has not changed.
 * @return {Component} the same component you passed in, with validation
 */
export default function withValidation(WrappedComponent) {
  return class ValidatedInput extends Component {
    constructor(props) {
      super(props)
      this.state = { status: null, message: null, value: props.value }
      // if there's no validation, auto-success
      if (!props.validate) this.state.status = 'success'
    }
    componentDidUpdate(prevProps, prevState) {
      // when validity status changes, reflect to parent if applicable
      if (this.state.status !== prevState.status) {
        this.props.onValidityChange &&
          this.props.onValidityChange(this.state.status, this.state.message)
      }

      // if forceValidate is changed to a truthy value, and the field has
      // not been validated, run validate
      if (
        this.props.forceValidate &&
        this.props.forceValidate !== prevProps.forceValidate &&
        this.state.status === null
      ) {
        this.validate(this.state.value)
      }
    }
    validate(value) {
      if (!value) value = ''
      const opt = this.props.validate
      this.setState({ value })

      // if it's a custom function, run it and use its return values
      // can return a promise if its async
      if (typeof opt === 'function') {
        const fn = opt(value)
        this.setState({ status: 'loading' })
        if (fn.then) {
          fn.then(res => {
            this.setState({
              status: res.status,
              message: res.message || null
            })
          })
        } else {
          this.setState({
            status: fn.status,
            message: fn.message || null
          })
        }
      }

      // if its a string preset, run the preset
      if (typeof opt === 'string') {
        if (opt === 'email') {
          if (value.match(/.+@.+\..+/)) {
            this.setState({ status: 'success', message: null })
          } else {
            this.setState({
              status: 'error',
              message: 'Please enter a valid email address'
            })
          }
        }

        if (opt === 'not-empty') {
          if (value.match(/.+/)) {
            this.setState({ status: 'success', message: null })
          } else {
            this.setState({
              status: 'error',
              message: 'This field cannot be empty'
            })
          }
        }

        if (opt === 'checked') {
          if (value) {
            this.setState({ status: 'success', message: null })
          } else {
            this.setState({
              status: 'error',
              message: 'You must accept in order to proceed.'
            })
          }
        }
      }

      // pass the value through to the parent's onValueChange if present
      if (this.props.onValueChange) {
        this.props.onValueChange(value)
      }
    }
    render() {
      // don't pass props meant only for the validation wrapper
      const dedupeProps = assign({}, this.props)
      delete dedupeProps.onValidityChange
      delete dedupeProps.validate

      // we hijack the valueChange function to run our validation, but call
      // the parent's function with the value as well
      delete dedupeProps.onValueChange

      return (
        <div class={`g-validation-container ${this.state.status}`}>
          <WrappedComponent
            onValueChange={this.validate.bind(this)}
            {...dedupeProps}
          />
          {this.props.validate && (
            <div
              class="message"
              dangerouslySetInnerHTML={{ __html: this.state.message }}
            />
          )}
        </div>
      )
    }
  }
}
